@@ -1,6 +1,6 @@
 /*
  *   stunnel       TLS offloading and load-balancing proxy
- *   Copyright (C) 1998-2020 Michal Trojnara <Michal.Trojnara@stunnel.org>
+ *   Copyright (C) 1998-2021 Michal Trojnara <Michal.Trojnara@stunnel.org>
  *
  *   This program is free software; you can redistribute it and/or modify it
  *   under the terms of the GNU General Public License as published by the
@@ -139,10 +139,16 @@ typedef long SSL_OPTIONS_TYPE;
 int context_init(SERVICE_OPTIONS *section) { /* init TLS context */
     /* create a new TLS context */
 #if OPENSSL_VERSION_NUMBER>=0x10100000L
-    if(section->option.client)
-        section->ctx=SSL_CTX_new(TLS_client_method());
-    else /* server mode */
-        section->ctx=SSL_CTX_new(TLS_server_method());
+#if OPENSSL_VERSION_NUMBER>=0x30000000L
+    section->ctx=SSL_CTX_new_ex(NULL,
+        EVP_default_properties_is_fips_enabled(NULL) ?
+            "fips=yes" : "provider!=fips",
+        section->option.client ?
+            TLS_client_method() : TLS_server_method());
+#else /* OPENSSL_VERSION_NUMBER<0x30000000L */
+    section->ctx=SSL_CTX_new(section->option.client ?
+        TLS_client_method() : TLS_server_method());
+#endif /* OPENSSL_VERSION_NUMBER>=0x30000000L */
     if(!SSL_CTX_set_min_proto_version(section->ctx,
             section->min_proto_version)) {
         s_log(LOG_ERR, "Failed to set the minimum protocol version 0x%X",
@@ -213,6 +219,11 @@ int context_init(SERVICE_OPTIONS *section) { /* init TLS context */
 
     /* TLS options: configure the stunnel defaults first */
     SSL_CTX_set_options(section->ctx, SSL_OP_NO_SSLv2|SSL_OP_NO_SSLv3);
+    /* no session ticket gets sent to the client at all in TLSv1.2
+       and below, but a stateful ticket will be sent in TLSv1.3 */
+    if(!section->option.client && !section->option.session_resume) {
+        SSL_CTX_set_options(section->ctx, SSL_OP_NO_TICKET);
+    }
 #ifdef SSL_OP_NO_COMPRESSION
     /* we implemented a better way to disable compression if needed */
     SSL_CTX_clear_options(section->ctx, SSL_OP_NO_COMPRESSION);
@@ -228,11 +239,12 @@ int context_init(SERVICE_OPTIONS *section) { /* init TLS context */
 
     /* TLS options: log the configured values */
 #if OPENSSL_VERSION_NUMBER>=0x009080dfL
-    s_log(LOG_DEBUG, "TLS options: 0x%08lX (+0x%08lX, -0x%08lX)",
+    s_log(LOG_DEBUG,
+        "TLS options: 0x%" PRIX64 " (+0x%" PRIX64 ", -0x%" PRIX64 ")",
         SSL_CTX_get_options(section->ctx),
         section->ssl_options_set, section->ssl_options_clear);
 #else /* OpenSSL older than 0.9.8m */
-    s_log(LOG_DEBUG, "TLS options: 0x%08lX (+0x%08lX)",
+    s_log(LOG_DEBUG, "TLS options: 0x%" PRIX64 " (+0x%" PRIX64 ")",
         SSL_CTX_get_options(section->ctx), section->ssl_options_set);
 #endif /* OpenSSL 0.9.8m or later */
 
@@ -268,14 +280,27 @@ int context_init(SERVICE_OPTIONS *section) { /* init TLS context */
         unsigned servname_len=(unsigned)strlen(section->servname);
         if(servname_len>SSL_MAX_SSL_SESSION_ID_LENGTH)
             servname_len=SSL_MAX_SSL_SESSION_ID_LENGTH;
+#ifndef OPENSSL_NO_TLS1_3
+        /* suppress all tickets (stateful and stateless) in TLSv1.3 */
+        if(!section->option.session_resume && !SSL_CTX_set_num_tickets(section->ctx, 0)) {
+            sslerror("SSL_CTX_set_num_tickets");
+            return 1; /* FAILED */
+        }
+#endif /* TLS 1.3 */
         if(!SSL_CTX_set_session_id_context(section->ctx,
                 (unsigned char *)section->servname, servname_len)) {
             sslerror("SSL_CTX_set_session_id_context");
             return 1; /* FAILED */
         }
     }
-    SSL_CTX_set_session_cache_mode(section->ctx,
-        SSL_SESS_CACHE_BOTH | SSL_SESS_CACHE_NO_INTERNAL_STORE);
+    if(section->option.session_resume) {
+        SSL_CTX_set_session_cache_mode(section->ctx,
+            SSL_SESS_CACHE_BOTH | SSL_SESS_CACHE_NO_INTERNAL_STORE);
+    } else {
+        SSL_CTX_set_session_cache_mode(section->ctx, SSL_SESS_CACHE_OFF);
+    }
+    s_log(LOG_INFO, "Session resumption %s", section->option.session_resume
+        ? "enabled" : "disabled");
     SSL_CTX_sess_set_cache_size(section->ctx, section->session_size);
     SSL_CTX_set_timeout(section->ctx, section->session_timeout);
     SSL_CTX_sess_set_new_cb(section->ctx, sess_new_cb);
@@ -302,7 +327,8 @@ int context_init(SERVICE_OPTIONS *section) { /* init TLS context */
     dh_init(section); /* ignore the result (errors are not critical) */
 #endif /* OPENSSL_NO_DH */
 #ifndef OPENSSL_NO_ECDH
-    ecdh_init(section); /* ignore the result (errors are not critical) */
+    if(ecdh_init(section))
+        return 1; /* FAILED */
 #endif /* OPENSSL_NO_ECDH */
 
     return 0; /* OK */
@@ -501,9 +527,7 @@ NOEXPORT int SSL_CTX_set1_groups_list(SSL_CTX *ctx, char *list) {
 NOEXPORT int ecdh_init(SERVICE_OPTIONS *section) {
     s_log(LOG_DEBUG, "ECDH initialization");
     if(!SSL_CTX_set1_groups_list(section->ctx, section->curves)) {
-#if OPENSSL_VERSION_NUMBER >= 0x10101000L
-        sslerror("SSL_CTX_set1_groups_list");
-#endif /* OpenSSL version >= 1.1.1 */
+        s_log(LOG_ERR, "Invalid groups list in 'curves'");
         return 1; /* FAILED */
     }
     s_log(LOG_DEBUG, "ECDH initialized with curves %s", section->curves);
@@ -885,6 +909,23 @@ NOEXPORT int load_cert_engine(SERVICE_OPTIONS *section) {
     return 0; /* OK */
 }
 
+UI_METHOD *ui_stunnel() {
+    static UI_METHOD *ui_method=NULL;
+
+    if(ui_method) /* already initialized */
+        return ui_method;
+    ui_method=UI_create_method("stunnel UI");
+    if(!ui_method) {
+        sslerror("UI_create_method");
+        return NULL;
+    }
+    UI_method_set_opener(ui_method, ui_get_opener());
+    UI_method_set_writer(ui_method, ui_get_writer());
+    UI_method_set_reader(ui_method, ui_get_reader());
+    UI_method_set_closer(ui_method, ui_get_closer());
+    return ui_method;
+}
+
 NOEXPORT int load_key_engine(SERVICE_OPTIONS *section) {
     int i;
     EVP_PKEY *pkey;
@@ -896,7 +937,7 @@ NOEXPORT int load_key_engine(SERVICE_OPTIONS *section) {
 
     for(i=0; i<3; i++) {
         pkey=ENGINE_load_private_key(section->engine, section->key,
-            UI_stunnel(), NULL);
+            ui_stunnel(), NULL);
         if(!pkey) {
             if(i<2 && ui_retry()) { /* wrong PIN */
                 sslerror_queue(); /* dump the error queue */
@@ -982,6 +1023,9 @@ NOEXPORT int ui_retry() {
         switch(ERR_GET_REASON(err)) {
         case UI_R_RESULT_TOO_LARGE:
         case UI_R_RESULT_TOO_SMALL:
+#ifdef UI_R_PROCESSING_ERROR
+        case UI_R_PROCESSING_ERROR:
+#endif
             return 1;
         default:
             return 0;
